;------------;
; Danmaku    ;
;------------;
; Initial XY position and direction set by generating sprite.

!DanmakuYPosLo		= $1E02
!DanmakuYPosHi		= $1E2A
!DanmakuXPosLo		= $1E16
!DanmakuXPosHi		= $1E3E
!DanmakuYSpeed		= $1E52
!DanmakuXSpeed		= $1E66
!DanmakuYSpeedFrac	= $1E7A
!DanmakuXSpeedFrac	= $1E8E
!DanmakuDir		= $7EC300
!DanmakuTimer		= $7EC314



DanmakuXSpeedTable:
db $00,$F0,$F0,$F0,$00,$10,$10,$10
DanmakuYSpeedTable:
db $10,$10,$00,$F0,$F0,$F0,$00,$10

DanmakuTilemap:
db $06,$08,$04,$08,$06,$08,$04,$08
DanmakuTileProperties:		;YXPPCCCT format, currently using palette E
db $BD,$FD,$7D,$7D,$3D,$3D,$3D,$BD

DanmakuOAM:
db $40,$44,$48,$4C,$50,$54,$58,$5C,$60,$64,$68,$6C,$80,$84,$88,$8C,$B0,$B4,$B8,$BC


DanmakuOffscreen:
SEP #$20		; 8 bit A
LDA #$00		; kill sprite
STA $1892,y
RTS

Main:

LDA !DanmakuYPosLo,y	; y pos (low)
STA $00			; store to scratch
LDA !DanmakuYPosHi,y	; y pos (high)
STA $01			; 
LDA !DanmakuXPosLo,y	; x pos (low)
STA $02			;
LDA !DanmakuXPosHi,y	; x pos (high)
STA $03

REP #$20		; 16 bit A
LDA $00			; sprite y
SEC
SBC $1C			; layer 1 y
BMI DanmakuOffscreen	; kill self if offscreen
CMP #$00F0
BCS DanmakuOffscreen
STA $00

LDA $02			; sprite x
SEC
SBC $1A			; layer 1 x
BMI DanmakuOffscreen	; kill self if offscreen
CMP #$00F0
BCS DanmakuOffscreen
STA $01
	
SEP #$20		; 8 bit A

DanmakuGFX:			; OAM routine starts here.
LDX.w DanmakuOAM,y 		; Get OAM index.
LDA $00				; \ Set y position of tile
STA $0201,x			; /
LDA $01				; \ Set x position of tile
STA $0200,x			; /
PHX				; \ Set tile to use
TYX				;  |
LDA !DanmakuDir,x		;  |
TAX				;  |
LDA DanmakuTilemap,x		;  |
PLX				;  |
STA $0202,x                     ; /
PHX				; \
TYX				;  |
LDA !DanmakuDir,x		;  |
TAX				;  |
LDA DanmakuTileProperties,x	;  | Set tile properties
PLX				;  |
STA $0203,x			; /
PHX
TXA
LSR
LSR
TAX
LDA #$02
STA $0420,x
PLX

LDA $9D						; Branch if sprites locked
BNE DanmakuReturn

JSR DanmakuCollision
BCS NotHit
PHY
JSL $00F5B7
PLY

NotHit:
TYX
LDA !DanmakuTimer,x				; Branch if not time for sprite to start moving
BEQ DanmakuCont
DEC A
STA !DanmakuTimer,x
BNE DanmakuReturn

DanmakuCont:
LDA !DanmakuDir,x
TAX
LDA DanmakuXSpeedTable,x
STA !DanmakuXSpeed,y
LDA DanmakuYSpeedTable,x
STA !DanmakuYSpeed,y

JSR DanmakuSpeed

DanmakuReturn:
RTS



DanmakuCollision:			; checks collision for 16x16 cluster sprites
					; pretty leniently too
PHX

LDA $94					;
SEC					; checks if mario's in range horizontally
SBC !DanmakuXPosLo,y			;
CLC					;
ADC #$0A				;
CMP #$14				;
BCS NoDanmakuContact			; return if not
LDA #$18
LDX $73					; mario is ducking flag
BNE ContactContinue			; collision box = 10 if so
LDX $19					; mario powerup
BEQ ContactContinue			; collision box = 10 if small
LDA #$24				; else collision box = 20

ContactContinue:
STA $00					; store to scratch
LDA $96					; mario y (screen)
SEC
SBC !DanmakuYPosLo,y			; sprite y
CLC
ADC #$1C				; offset by 10 (so as to not have overflow issues; was offset earlier too
CMP $00
					; carry = clear (positive return)
NoDanmakuContact:
PLX
RTS



DanmakuSpeed:		
LDA !DanmakuYSpeed,y	; sprite y speed
ASL
ASL
ASL
ASL
CLC
ADC !DanmakuYSpeedFrac,y	; accumulating fraction bits for y speed
STA !DanmakuYSpeedFrac,y	; set it
PHP
LDA !DanmakuYSpeed,y	; sprite y speed
LSR
LSR
LSR
LSR
CMP #$08
LDX #$00
BCC ASpeedLabel
ORA #$F0
DEX

ASpeedLabel:
PLP
ADC !DanmakuYPosLo,y	; sprite y (low)
STA !DanmakuYPosLo,y	; set it
TXA
ADC !DanmakuYPosHi,y	; sprite y (high)
STA !DanmakuYPosHi,y	; set it

	; do it all again for x speeds

LDA !DanmakuXSpeed,y	; sprite x speed
ASL
ASL
ASL
ASL
CLC
ADC !DanmakuXSpeedFrac,y	; accumulating fraction bits for x speed
STA !DanmakuXSpeedFrac,y	; set it
PHP
LDA !DanmakuXSpeed,y	; sprite x speed
LSR
LSR
LSR
LSR
CMP #$08
LDX #$00
BCC AnotherSpeedLabel
ORA #$F0
DEX

AnotherSpeedLabel:
PLP
ADC !DanmakuXPosLo,y	; sprite x (low)
STA !DanmakuXPosLo,y	; set it
TXA
ADC !DanmakuXPosHi,y	; sprite x (high)
STA !DanmakuXPosHi,y	; set it

RTS		; return